#!/usr/bin/env python

import doctest
import inspect

# This class is used to monkeypatch doctest, which fails to recognize doctests
# in extension modules.  The single change is to add an isbuiltin() call in the
# recursion loop, since functions defined in extension modules are seen by
# inspect as 'builtins' (though they are not).

# Hold a reference to the original, which we'll need since we'll be
# monkeypatching doctest and would otherwise get an infinite recursion.
DocTestFinderOri = doctest.DocTestFinder

class DocTestFinder(doctest.DocTestFinder):

   def _find(self, tests, obj, name, module, source_lines, globs, seen):
        """
        Find tests for the given object and any contained objects, and
        add them to `tests`.
        """

        DocTestFinderOri._find(self,tests, obj, name, module,
        source_lines, globs, seen)

        # Look for tests in a module's contained objects.
        if inspect.ismodule(obj) and self._recurse:
            for valname, val in obj.__dict__.items():
                valname = '%s.%s' % (name, valname)
                # Recurse to functions & classes.
                if ((inspect.isfunction(val) or inspect.isclass(val) or
                     inspect.isbuiltin(val)) and
                    self._from_module(module, val)):
                    self._find(tests, val, valname, module, source_lines,
                               globs, seen)

if __name__ == "__main__":
    # Simple test with a cython-built extension that shows the difference
    import primes

    print '*************** Before monkeypatching doctest *****************'
    doctest.testmod(primes,verbose=True)

    # Monkeypatch doctest with our fixed class
    doctest.DocTestFinder = DocTestFinder

    print '*************** After monkeypatching doctest *****************'
    doctest.master = None  # reset a module global in doctest for reporting
    doctest.testmod(primes,verbose=True)

